import json
import os

import mock
import unittest

from lpreports.launchpad import client
from lpreports.launchpad import consts


class LaunchpadClientTest(unittest.TestCase):

    def _load_fixture(self, file_name):
        path = os.path.join(os.path.dirname(__file__),
                            "fixtures", file_name)
        with open(path) as fp:
            return json.load(fp)

    def setUp(self):
        super(LaunchpadClientTest, self).setUp()
        self.lp = client.LaunchpadClient(None, None, None)

    def test_auth_headers(self):
        consumer_key = "consumer ( ) key"
        token = "token"
        signature = "signature"

        lp = client.LaunchpadClient(consumer_key, token, signature)
        headers = lp._create_auth_headers()
        auth_header = headers["Authorization"]

        self.assertIn(consumer_key, auth_header)
        self.assertIn(token, auth_header)
        self.assertIn(signature, auth_header)

        agent_header = headers["user-agent"]
        self.assertIn(consumer_key, agent_header)

    def test_process_lp_date(self):
        date = self.lp._process_lp_date("2014-08-15T17:06:21.439085+00:00")
        self.assertEqual(2014, date.year)
        self.assertEqual(8, date.month)
        self.assertEqual(15, date.day)

        self.assertEqual(17, date.hour)
        self.assertEqual(6, date.minute)

    @mock.patch("lpreports.launchpad.client.LaunchpadClient._get_json_from_url")
    def test_get_verifier(self, fn):
        fn.return_value = self._load_fixture("bug_activity.json")

        self.assertEqual("https://api.launchpad.net/1.0/~kromanenko",
                         self.lp.get_bug_verifier("mos/6.0.x", "fixtured"))

        self.assertEqual("https://api.launchpad.net/1.0/~vsedelnik",
                         self.lp.get_bug_verifier("mos/6.0-updates", "fixtured"))

    @mock.patch("lpreports.launchpad.client.LaunchpadClient.get_bug_by_url")
    @mock.patch("lpreports.launchpad.client.LaunchpadClient.get_bug_verifier")
    def test_process_bug_task(self, get_bug_verifier, get_bug_by_url):
        get_bug_by_url.return_value = self._load_fixture("bug.json")
        get_bug_verifier.return_value = \
            "https://api.launchpad.net/1.0/~kromanenko"

        raw = self._load_fixture("bug_task.json")
        task = self.lp.process_bug_task(raw)

        self.assertEqual("ativelkov", task["assignee"])
        self.assertEqual("fsoppelsa", task["owner"])
        self.assertEqual("Critical", task["importance"])
        self.assertEqual("mos", task["target_name"])
        self.assertEqual("6.1", task["milestone"])
        self.assertSequenceEqual(["customer-found", "glance", "swift"],
                                 task["tags"])
        self.assertEqual(False, task["private"])
        self.assertEqual("Public", task["information_type"])

        date_fields = [k for k in task.keys() if k.startswith("date_")]
        self.assertEqual(sorted(consts.DATE_FIELDS),
                         sorted(date_fields))

    @mock.patch("lpreports.launchpad.client.LaunchpadClient._get_json_from_url")
    def test_read_chunks(self, get_json):
        chunk_1 = self._load_fixture("chunk_1.json")
        chunk_2 = self._load_fixture("chunk_2.json")
        related_task = self._load_fixture("bug_task.json")
        related_task_url = "https://api.launchpad.net/1.0/mos/+bug/" \
                           "1157261/related_tasks"

        def side_effect(url):
            if "searchTasks" in url and "ws.size" not in url:
                return chunk_1
            elif "searchTasks" in url and "ws.size" in url:
                return chunk_2
            elif related_task_url == url:
                return {"entries": [related_task]}
            elif "related_tasks" in url:
                return {"entries": []}

        get_json.side_effect = side_effect

        tasks = [x for x in self.lp.find_bug_tasks("mos", 0)]

        # len(chunk_1) = 3, len(chunk_2) = 3, + 1 related task
        self.assertEqual(7, len(tasks))
