import datetime
import unittest

import mock
import pytz

import lpreports.launchpad.bug as bug
import lpreports.launchpad.release_chart as rc
import lpreports.utils.common as utils


BUGS = [
    {u'assignee': None,
     u'cves': [],
     u'date_assigned': None,
     u'date_closed': None,
     u'date_confirmed': datetime.datetime(2015, 11, 9, 8, 9, 59, 604000, tzinfo=pytz.UTC),
     u'date_created': datetime.datetime(2015, 11, 9, 8, 4, 40, 576000, tzinfo=pytz.UTC),
     u'date_fix_committed': datetime.datetime(2015, 12, 11, 12, 22, 28, 259000, tzinfo=pytz.UTC),
     u'date_fix_released': None,
     u'date_in_progress': datetime.datetime(2015, 11, 11, 8, 9, 59, 604000, tzinfo=pytz.UTC),
     u'date_incomplete': None,
     u'date_last_updated': datetime.datetime(2015, 11, 11, 12, 22, 29, 732000, tzinfo=pytz.UTC),
     u'date_left_closed': None,
     u'date_left_new': datetime.datetime(2015, 11, 9, 8, 9, 59, 604000, tzinfo=pytz.UTC),
     u'date_triaged': datetime.datetime(2015, 11, 9, 8, 9, 59, 604000, tzinfo=pytz.UTC),
     u'id': 1525111,
     u'importance': u'Undecided',
     u'information_type': u'Public',
     u'milestone': None,
     u'owner': u'vsaienko',
     u'private': False,
     u'status': u'Fix Committed',
     u'tags': [],
     u'target_name': u'mos',
     u'title': u'[Iornic] Errors and tracebacks are hiden in runtests.sh',
     u'verifier': None},
    {u'assignee': u'enikanorov',
     u'cves': [],
     u'date_assigned': datetime.datetime(2015, 11, 12, 11, 5, 11, 551000, tzinfo=pytz.UTC),
     u'date_closed': None,
     u'date_confirmed': datetime.datetime(2015, 11, 12, 11, 10, 10, 92000, tzinfo=pytz.UTC),
     u'date_created': datetime.datetime(2015, 11, 12, 11, 4, 38, 910000, tzinfo=pytz.UTC),
     u'date_fix_committed': None,
     u'date_fix_released': None,
     u'date_in_progress': datetime.datetime(2015, 11, 12, 11, 10, 10, 92000, tzinfo=pytz.UTC),
     u'date_incomplete': None,
     u'date_last_updated': datetime.datetime(2015, 11, 12, 11, 10, 10, 426000, tzinfo=pytz.UTC),
     u'date_left_closed': None,
     u'date_left_new': datetime.datetime(2015, 11, 12, 11, 10, 10, 92000, tzinfo=pytz.UTC),
     u'date_triaged': datetime.datetime(2015, 11, 12, 11, 10, 10, 92000, tzinfo=pytz.UTC),
     u'id': 1525495,
     u'importance': u'High',
     u'information_type': u'Public',
     u'milestone': u'6.1-mu-5',
     u'owner': u'enikanorov',
     u'private': False,
     u'status': u'In Progress',
     u'tags': [],
     u'target_name': u'mos',
     u'title': u"DHCP/L3 agent may change status of ports it doesn't own",
     u'verifier': None},
    {u'assignee': None,
     u'cves': [],
     u'date_assigned': None,
     u'date_closed': datetime.datetime(2015, 11, 28, 11, 13, 24, 24000, tzinfo=pytz.UTC),
     u'date_confirmed': datetime.datetime(2015, 11, 28, 11, 13, 24, 24000, tzinfo=pytz.UTC),
     u'date_created': datetime.datetime(2015, 11, 27, 11, 11, 13, 989000, tzinfo=pytz.UTC),
     u'date_fix_committed': datetime.datetime(2015, 11, 29, 11, 13, 24, 24000, tzinfo=pytz.UTC),
     u'date_fix_released': datetime.datetime(2015, 11, 30, 11, 13, 24, 24000, tzinfo=pytz.UTC),
     u'date_in_progress': datetime.datetime(2015, 11, 28, 11, 13, 24, 24000, tzinfo=pytz.UTC),
     u'date_incomplete': None,
     u'date_last_updated': datetime.datetime(2015, 11, 28, 11, 13, 24, 443000, tzinfo=pytz.UTC),
     u'date_left_closed': None,
     u'date_left_new': datetime.datetime(2015, 11, 28, 11, 13, 24, 24000, tzinfo=pytz.UTC),
     u'date_triaged': datetime.datetime(2015, 11, 28, 11, 13, 24, 24000, tzinfo=pytz.UTC),
     u'id': 1525223,
     u'importance': u'Undecided',
     u'information_type': u'Public',
     u'milestone': None,
     u'owner': u'oklochkova',
     u'private': False,
     u'status': u'Fix Released',
     u'tags': [u'horizon'],
     u'target_name': u'mos',
     u'title': u'Failed to Launch Instance. "Danger: There was an error submitting the form. Please try again."',
     u'verifier': u'oklochkova'},
    {u'assignee': None,
     u'cves': [],
     u'date_assigned': None,
     u'date_closed': None,
     u'date_confirmed': datetime.datetime(2015, 12, 5, 16, 2, 23, 848000, tzinfo=pytz.UTC),
     u'date_created': datetime.datetime(2015, 12, 5, 1, 54, 10, 627000, tzinfo=pytz.UTC),
     u'date_fix_committed': None,
     u'date_fix_released': None,
     u'date_in_progress': None,
     u'date_incomplete': datetime.datetime(2015, 12, 5, 16, 2, 23, 848000, tzinfo=pytz.UTC),
     u'date_last_updated': datetime.datetime(2015, 12, 5, 17, 10, 3, 256000, tzinfo=pytz.UTC),
     u'date_left_closed': None,
     u'date_left_new': datetime.datetime(2015, 12, 5, 16, 2, 23, 848000, tzinfo=pytz.UTC),
     u'date_triaged': datetime.datetime(2015, 12, 5, 16, 2, 23, 848000, tzinfo=pytz.UTC),
     u'id': 1526363,
     u'importance': u'Undecided',
     u'information_type': u'Private Security',
     u'milestone': None,
     u'owner': u'dlapshin',
     u'private': True,
     u'status': u'Incomplete',
     u'tags': [],
     u'target_name': u'mos',
     u'title': u'Certain ports are accessible from external network',
     u'verifier': None},
    {u'assignee': u'esikachev',
     u'cves': [],
     u'date_assigned': datetime.datetime(2015, 12, 17, 13, 52, 32, 932000, tzinfo=pytz.UTC),
     u'date_closed': None,
     u'date_confirmed': datetime.datetime(2015, 12, 17, 14, 12, 15, 6000, tzinfo=pytz.UTC),
     u'date_created': datetime.datetime(2015, 12, 17, 13, 52, 32, 596000, tzinfo=pytz.UTC),
     u'date_fix_committed': None,
     u'date_fix_released': None,
     u'date_in_progress': datetime.datetime(2015, 12, 17, 14, 12, 15, 6000, tzinfo=pytz.UTC),
     u'date_incomplete': None,
     u'date_last_updated': datetime.datetime(2015, 12, 17, 14, 12, 20, 376000, tzinfo=pytz.UTC),
     u'date_left_closed': None,
     u'date_left_new': datetime.datetime(2015, 12, 17, 14, 12, 15, 6000, tzinfo=pytz.UTC),
     u'date_triaged': datetime.datetime(2015, 12, 17, 14, 12, 15, 6000, tzinfo=pytz.UTC),
     u'id': 1527264,
     u'importance': u'Undecided',
     u'information_type': u'Public',
     u'milestone': u'8.0',
     u'owner': u'esikachev',
     u'private': False,
     u'status': u'In Progress',
     u'tags': [u'ostf', u'sahara'],
     u'target_name': u'mos',
     u'title': u'[Sahara] Incorrect order of steps in Sahara service tests',
     u'verifier': None},
    {u'assignee': u'mos-ceilometer',
     u'cves': [],
     u'date_assigned': datetime.datetime(2015, 12, 18, 15, 26, 31, 872000, tzinfo=pytz.UTC),
     u'date_closed': None,
     u'date_confirmed': datetime.datetime(2015, 12, 18, 15, 57, 36, 267000, tzinfo=pytz.UTC),
     u'date_created': datetime.datetime(2015, 12, 18, 15, 25, 48, 676000, tzinfo=pytz.UTC),
     u'date_fix_committed': None,
     u'date_fix_released': None,
     u'date_in_progress': datetime.datetime(2015, 12, 18, 15, 57, 36, 267000, tzinfo=pytz.UTC),
     u'date_incomplete': None,
     u'date_last_updated': datetime.datetime(2015, 12, 18, 15, 57, 36, 839000, tzinfo=pytz.UTC),
     u'date_left_closed': None,
     u'date_left_new': datetime.datetime(2015, 12, 18, 15, 57, 36, 267000, tzinfo=pytz.UTC),
     u'date_triaged': datetime.datetime(2015, 12, 18, 15, 57, 36, 267000, tzinfo=pytz.UTC),
     u'id': 1527651,
     u'importance': u'Undecided',
     u'information_type': u'Public',
     u'milestone': u'8.0',
     u'owner': u'idegtiarov',
     u'private': False,
     u'status': u'In Progress',
     u'tags': [],
     u'target_name': u'mos',
     u'title': u'mandatory limit is not mentioned in help cli docs',
     u'verifier': None}
]


def _get_bugs_fake(project_name, statuses, milestones):
    return [bug.Bug(b) for b in BUGS
            if b['status'] in statuses]


class TestIncomingOutgoing(unittest.TestCase):
    @mock.patch('lpreports.db.get_bugs')
    def test_empty_bugs_list(self, get_bugs_mock):
        get_bugs_mock.return_value = []
        chart = rc.ReleaseChart('mos', '7.0')

        actual = chart.get_incoming_outgoing_data()
        expected = [
            {"key": "Incoming", "values": []},
            {"key": "Outgoing", "values": []}
        ]

        self.assertEqual(actual, expected)

    @mock.patch('lpreports.db.get_bugs')
    def test_default_tickday(self, get_bugs_mock):
        get_bugs_mock.side_effect = _get_bugs_fake
        chart = rc.ReleaseChart('mos', '7.0')

        actual = chart.get_incoming_outgoing_data()
        expected = [
            {"key": "Incoming",
             "values": [[1447027200000, 2],
                         [1448236800000, 1],
                         [1448841600000, 1],
                         [1449446400000, 0],
                         [1450051200000, 2]]},
            {"key": "Outgoing",
             "values": [[1447027200000, 0],
                        [1448236800000, 1],
                        [1448841600000, 0],
                        [1449446400000, 1],
                        [1450051200000, 0]]}
        ]

        self.assertEqual(actual, expected)

    @mock.patch('lpreports.db.get_bugs')
    def test_different_tickday(self, get_bugs_mock):
        get_bugs_mock.side_effect = _get_bugs_fake
        chart = rc.ReleaseChart('mos', '7.0')

        actual = chart.get_incoming_outgoing_data(utils.Weekday.WEDNESDAY)
        expected = [
            {"key": "Incoming",
             "values": [[1446595200000, 1],
                        [1447200000000, 1],
                        [1448409600000, 1],
                        [1449014400000, 1],
                        [1449619200000, 0],
                        [1450224000000, 2]]},
            {"key": "Outgoing",
             "values": [[1446595200000, 0],
                        [1447200000000, 0],
                        [1448409600000, 1],
                        [1449014400000, 0],
                        [1449619200000, 1],
                        [1450224000000, 0]]},
        ]

        self.assertEqual(actual, expected)
